//
// Copyright 2016 frankmuenster.de
//
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
//
//   http://ec.europa.eu/idabc/eupl
//
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//

package de.frankmuenster.hibiscus.mahoe;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;

import de.frankmuenster.hibiscus.mahoe.bcee.Bcee;
import de.frankmuenster.hibiscus.mahoe.bsh.SchwaebischHall;
import de.frankmuenster.hibiscus.mahoe.santander.SantanderBank;
import de.frankmuenster.hibiscus.mahoe.targobank.TargoBank;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteException;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.KontoType;
import de.willuhn.jameica.hbci.synchronize.AbstractSynchronizeBackend;
import de.willuhn.jameica.hbci.synchronize.jobs.SynchronizeJob;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.jameica.system.OperationCanceledException;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;
import de.willuhn.util.ProgressMonitor;

/**
 * Das Synchronisierungs-Backend der MahoeMain.
 *
 * @author Frank Mnster
 *
 */
public class MahoeSynchronizeBackend extends AbstractSynchronizeBackend<MahoeSynchronizeJobProvider> {

	/**
	 * Backend Klassen Name
	 */
	public final static String BACKEND_NAME = "de.frankmuenster.hibiscus.mahoe.MahoeSynchronizeBackend"; //$NON-NLS-1$

	/**
	 * Version des Kontos. Kann fr Updateprozesse verwendet werden.
	 */
	public static final String KONTO_VERSION = "KtoVersion"; //$NON-NLS-1$

	/**
	 * Property name fr die Kreditkartennummer
	 */
	public static final String CARD_NUMBER = "Kartennummer"; //$NON-NLS-1$

	/**
	 * Property name fr das Kreditkartenkonto
	 */
	public static final String CARD_ACCOUNT = "Kartenkonto"; //$NON-NLS-1$

	/**
	 * Property name fr den Kreditkarten-Anbieter
	 */
	public static final String CARD_COMPANY = "Anbieter"; //$NON-NLS-1$

	/**
	 * Kontoauszge speichern
	 */
	public static final String SAVE_PDF = "PDF speichern"; //$NON-NLS-1$
	/**
	 * Speicherpfad fr PDF Kontoauszge
	 */
	public static final String SAVE_PDF_PATH = "Speicherpfad"; //$NON-NLS-1$

	/**
	 * Der Bank Name, im Moment nur fr Santander Banken
	 */
	public static final String BANK_NAME = "Name der Bank"; //$NON-NLS-1$

	/**
	 * VTN fr Schwbisch Hall Vertrge
	 */
	public static final String BSH_VTN = "Vertragsnummer (VTN)"; //$NON-NLS-1$

	/**
	 * Die vom Job-Provider explizit untersttzten Bankleitzahlen.
	 */
	public final static ConcurrentHashMap<String, String> BLZ_SUPPORTED = new ConcurrentHashMap<String, String>();

	/**
	 * Ausgleichsbunchung anlegen
	 */
	public static final String CREATE_CLEARING = "Ausgleichsbuchung ?"; //$NON-NLS-1$

	/**
	 * Konto f&uuml; die Ausgleichsbuchung
	 */
	public static final String CLEARING_ACCOUNT = "Konto-ID der Buchung"; //$NON-NLS-1$

	/**
	 * Fgt eine BLZ zu den Untersttzen BLZ hinzu
	 *
	 * @param blz
	 * @param bankName
	 */
	public static void addBlzSupported(final String blz, final String bankName) {
		{
			MahoeSynchronizeBackend.BLZ_SUPPORTED.putIfAbsent(blz, bankName);
		}
	}

	/**
	 * Standardkonstruktor
	 */
	public MahoeSynchronizeBackend() {
		super();
	}

	/**
	 * @see de.willuhn.jameica.hbci.synchronize.SynchronizeBackend#getName()
	 */
	@Override
	public String getName() {
		return "Mahoe"; //$NON-NLS-1$
	}

	/**
	 * @see de.willuhn.jameica.hbci.synchronize.AbstractSynchronizeBackend#getJobProviderInterface()
	 */
	@Override
	protected Class<MahoeSynchronizeJobProvider> getJobProviderInterface() {
		return MahoeSynchronizeJobProvider.class;
	}

	/**
	 * @see de.willuhn.jameica.hbci.synchronize.AbstractSynchronizeBackend#createJobGroup(de.willuhn.jameica.hbci.rmi.Konto)
	 */
	@Override
	protected AbstractSynchronizeBackend<MahoeSynchronizeJobProvider>.JobGroup createJobGroup(final Konto konto) {
		return new MahoeJobGroup(konto);
	}

	/**
	 * @see de.willuhn.jameica.hbci.synchronize.AbstractSynchronizeBackend#getPropertyNames(de.willuhn.jameica.hbci.rmi.Konto)
	 */
	@Override
	public List<String> getPropertyNames(final Konto konto) {

		List<String> result = null;

		try {
			if (konto != null && !(konto.hasFlag(Konto.FLAG_DISABLED))) {

				if (SantanderBank.BLZ.equals(konto.getBLZ())) {
					result = new ArrayList<String>();
					result.add(MahoeSynchronizeBackend.BANK_NAME);
					final KontoType kontoType = KontoType.find(konto.getAccountType());
					if (KontoType.KREDITKARTE.equals(kontoType)) {
						result.add(MahoeSynchronizeBackend.CARD_NUMBER);
						result.add(MahoeSynchronizeBackend.CARD_COMPANY);
						result.add(MahoeSynchronizeBackend.SAVE_PDF);
						result.add(MahoeSynchronizeBackend.SAVE_PDF_PATH);
						result.add(MahoeSynchronizeBackend.CREATE_CLEARING);
						result.add(MahoeSynchronizeBackend.CLEARING_ACCOUNT);
					}
				} else if (TargoBank.BLZ.equals(konto.getBLZ())) {
					result = new ArrayList<String>();
					result.add(MahoeSynchronizeBackend.CARD_ACCOUNT);
					result.add(MahoeSynchronizeBackend.CARD_NUMBER);
					result.add(MahoeSynchronizeBackend.CARD_COMPANY);
					result.add(MahoeSynchronizeBackend.SAVE_PDF);
					result.add(MahoeSynchronizeBackend.SAVE_PDF_PATH);
				} else if (SchwaebischHall.BLZ.equals(konto.getBLZ())) {
					result = new ArrayList<String>();
					result.add(MahoeSynchronizeBackend.BSH_VTN);
				} else if (Bcee.BIC.equals(konto.getBic())) { // NOPMD
				}
			}
		} catch (RemoteException re) {
			Logger.error(I18N.tr("MahoeSynchronizeBackend.propertyNameNotFound"), re); //$NON-NLS-1$
		}
		return result;
	}

	/**
	 * @see de.willuhn.jameica.hbci.synchronize.AbstractSynchronizeBackend#supports(java.lang.Class,
	 *      de.willuhn.jameica.hbci.rmi.Konto)
	 */
	@Override
	public boolean supports(final Class<? extends SynchronizeJob> type, final Konto konto) {

		boolean support = false;

		if (super.supports(type, konto)) {
			try {

				Logger.debug("Support BLZ: " + konto.getBLZ()); //$NON-NLS-1$
				Logger.debug(I18N.tr("MahoeSynchronizeBackend.jobType") + type.getName()); //$NON-NLS-1$
				// Checken, ob wir die Bank untersttzen
				if (MahoeSynchronizeBackend.BLZ_SUPPORTED.containsKey(konto.getBLZ())) {
					support = true;
				}
			} catch (RemoteException re) {
				Logger.error(I18N.tr("MahoeSynchronizeBackend.checkSupportedFailed"), re); //$NON-NLS-1$
			}
		}
		return support;
	}

	/**
	 * Hier ist die Implementierung unseres Jobs
	 *
	 * @author Frank Mnster
	 */
	@SuppressWarnings({ "PMD.StdCyclomaticComplexity", "PMD.CyclomaticComplexity" })
	protected class MahoeJobGroup extends JobGroup {

		/**
		 * @param konto
		 *            Das Konto
		 */
		protected MahoeJobGroup(final Konto konto) {
			super(konto);
		}

		/**
		 * @throws WebSiteException
		 * @see de.willuhn.jameica.hbci.synchronize.AbstractSynchronizeBackend.JobGroup#sync()
		 */
		@Override
		protected void sync() throws Exception { // NOPMD

			final ProgressMonitor monitor = MahoeSynchronizeBackend.this.worker.getMonitor();
			final String ktoLang = this.getKonto().getLongName();
			final int step = 100 / MahoeSynchronizeBackend.this.worker.getSynchronization().size();

			monitor.log(" "); //$NON-NLS-1$
			monitor.log(I18N.tr("MahoeSynchronizeBackend.syncAccountMsg", ktoLang)); //$NON-NLS-1$
			Logger.info(I18N.tr("MahoeSynchronizeBackend.syncAccountMsg", ktoLang)); //$NON-NLS-1$

			int jobsCount = 0;
			// Java, OS und HTMLUnit Versionen
			final String text = "Java: " //$NON-NLS-1$
					+ java.lang.System.getProperty("java.version") + " | OS: " //$NON-NLS-1$ //$NON-NLS-2$
					+ java.lang.System.getProperty("os.name") + " " //$NON-NLS-1$ //$NON-NLS-2$
					+ java.lang.System.getProperty("os.arch") + " | HTMLUnit: " //$NON-NLS-1$ //$NON-NLS-2$
					+ com.gargoylesoftware.htmlunit.Version.getProductVersion() + " | System-Encoding : " //$NON-NLS-1$
					+ java.lang.System.getProperty("file.encoding") + " | Mahoe: " //$NON-NLS-1$ //$NON-NLS-2$
					+ Application.getPluginLoader().getPlugin(MahoeMain.class).getManifest().getVersion();
			Logger.info(text);
			monitor.log(text);

			for (final SynchronizeJob job : this.jobs) {
				this.checkInterrupted();
				jobsCount++;
				try {
					((InterfaceMahoeSynchronizeJob) job).execute(monitor, jobsCount == this.jobs.size());
				} catch (OperationCanceledException e) {
					Logger.debug(e.getMessage());
					throw e;
				} catch (WebSiteException e) {
					switch (e.getType()) {
					case LOGIN_FAILED:
						String msg = I18N.tr("MahoeSynchronizeBackend.loginFailed"); //$NON-NLS-1$
						Logger.error(msg, e);
						Application.getCallback().notifyUser(msg);
						StatusBarMessage msgUser = new StatusBarMessage( // NOPMD
								I18N.tr(msg), StatusBarMessage.TYPE_INFO);
						Application.getMessagingFactory().sendMessage(msgUser);
						monitor.setStatusText(msg);
						throw e;
					case LOGIN_ERROR:
						msg = I18N.tr("MahoeSynchronizeBackend.loginError"); //$NON-NLS-1$
						Logger.error(msg, e);
						Application.getCallback().notifyUser(msg);
						monitor.setStatusText(msg);
						throw e;
					case SERVICE_UNAVAILABLE:
						msg = I18N.tr("MahoeSynchronizeBackend.serviceUnavailable"); //$NON-NLS-1$
						Logger.error(msg, e);
						Application.getCallback().notifyUser(msg);
						msgUser = new StatusBarMessage(msg, // NOPMD
								StatusBarMessage.TYPE_ERROR); // NOPMD
						Application.getMessagingFactory().sendMessage(msgUser);
						monitor.setStatusText(I18N.tr("MahoeSynchronizeBackend.serviceUnavailable")); //$NON-NLS-1$
						monitor.setStatus(ProgressMonitor.STATUS_ERROR);
						throw e;
					case TRANSFER_FAILED:
						msg = e.getMessage(); // $NON-NLS-1$
						Logger.error(msg, e);
						Application.getCallback().notifyUser(msg);
						msgUser = new StatusBarMessage(msg, // NOPMD
								StatusBarMessage.TYPE_ERROR); // NOPMD
						Application.getMessagingFactory().sendMessage(msgUser);
						monitor.setStatusText("berweisung konnte nicht eingereicht werden."); //$NON-NLS-1$
						monitor.setStatus(ProgressMonitor.STATUS_ERROR);
						throw e;

					default:
						Logger.error(I18N.tr("MahoeSynchronizeBackend.getTransactionsFailedLog", //$NON-NLS-1$
								e.getMessage()), e);
						msg = I18N.tr("MahoeSynchronizeBackend.getTransactionsFailedMsg"); //$NON-NLS-1$
						msgUser = new StatusBarMessage(msg, // NOPMD
								StatusBarMessage.TYPE_ERROR); // NOPMD
						Application.getMessagingFactory().sendMessage(msgUser);

						throw e;
					}

				} catch (ApplicationException e) {
					Logger.error(e.getMessage(), e);
					throw e;
				} catch (Exception e) { // NOPMD
					Logger.error(I18N.tr("MahoeSynchronizeBackend.generalExpectionOccured"), e); //$NON-NLS-1$
					throw new ApplicationException(I18N.tr("MahoeSynchronizeBackend.generalExpectionOccured"), e); //$NON-NLS-1$
				}
				monitor.addPercentComplete(step);
			}
		}
	}

}
