//
// Copyright 2017 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.assistant;

import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.willuhn.jameica.gui.dialogs.BackgroundTaskDialog;
import de.willuhn.jameica.gui.util.SWTUtil;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.plugin.Plugin;
import de.willuhn.jameica.system.Application;
import de.willuhn.jameica.system.BackgroundTask;
import de.willuhn.jameica.system.OperationCanceledException;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;
import de.willuhn.util.ProgressMonitor;

/**
 * Abstrakter KonteMigrator. Stellt die Plugin-Deinstallation zur Verfgung.
 * 
 * @author Frank Mnster
 * 
 */
public abstract class AbstractAccountMigrator implements AccountMigrator {

	/**
	 * Standardkonstruktor
	 */
	public AbstractAccountMigrator() {
		// Absichtlich leer
	}

	/**
	 * @see AccountMigrator#migrate(Plugin)
	 */
	@Override
	public abstract void migrate(final Plugin plugin) throws ApplicationException;

	/**
	 * @see AccountMigrator#migrate(Plugin)
	 */
	@Override
	public abstract void update(final Konto account) throws ApplicationException;

	/**
	 * Entfernt ein PLugin
	 * 
	 * @param plugin
	 * @throws ApplicationException
	 */
	protected void uninstallPlugin(final Plugin plugin) throws ApplicationException {
		final BackgroundTask task = new BackgroundTask() {

			/**
			 * @see de.willuhn.jameica.system.BackgroundTask#run(de.willuhn.util.ProgressMonitor)
			 */
			@Override
			public void run(final ProgressMonitor monitor) throws ApplicationException {
				Application.getPluginLoader().unInstall(plugin.getManifest(), true, monitor);
			}

			/**
			 * @see de.willuhn.jameica.system.BackgroundTask#isInterrupted()
			 */
			@Override
			public boolean isInterrupted() {
				return false;
			}

			/**
			 * @see de.willuhn.jameica.system.BackgroundTask#interrupt()
			 */
			@Override
			public void interrupt() {
				// Nix zu tun hier
			}
		};

		try {
			final BackgroundTaskDialog bgTaskDialog = new BackgroundTaskDialog(BackgroundTaskDialog.POSITION_CENTER,
					task);
			bgTaskDialog.setSideImage(SWTUtil.getImage("user-trash-full.png")); //$NON-NLS-1$
			bgTaskDialog.setTitle(I18N.tr("AbstractAccountMigrator.deinstallDialogTitle")); //$NON-NLS-1$
			bgTaskDialog.setPanelText(I18N.tr("AbstractAccountMigrator.deinstallDialogPanelText")); //$NON-NLS-1$
			bgTaskDialog.open();
		} catch (ApplicationException ae) { // NOPMD
			throw ae;
		} catch (OperationCanceledException oce) {
			Logger.info(I18N.tr("AbstractAccountMigrator.operationCancelled")); //$NON-NLS-1$
		} catch (Exception e) { // NOPMD
			Logger.error(I18N.tr("AbstractAccountMigrator.deinstallationFailedLog"), e); //$NON-NLS-1$
			throw new ApplicationException(I18N.tr("AbstractAccountMigrator.deinstallationFailedMsg", e.getMessage()), //$NON-NLS-1$
					e);
		}
	}

}