//
// Copyright 2016 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.bcee.jobs;

import java.rmi.RemoteException;
import java.util.Arrays;

import de.frankmuenster.hibiscus.mahoe.AbstractMahoeSynchronizeJobSepaUeberweisung;
import de.frankmuenster.hibiscus.mahoe.bcee.Bcee;
import de.frankmuenster.hibiscus.mahoe.bcee.web.BceeWebAccountInformation;
import de.frankmuenster.hibiscus.mahoe.bcee.web.BceeBankMessages;
import de.frankmuenster.hibiscus.mahoe.bcee.web.BceeWebSite;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.frankmuenster.hibiscus.mahoe.utils.PasswdCache;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteException;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteExceptionTypes;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Der Job der die eigentliche Arbeit macht.
 * 
 * @author Frank Mnster
 * 
 */
public class BceeSynchronizeJobSepaUeberweisung
		extends AbstractMahoeSynchronizeJobSepaUeberweisung<BceeWebAccountInformation, BceeBankMessages> {

	/**
	 * Standartkonstruktor
	 */
	public BceeSynchronizeJobSepaUeberweisung() {
		super();
	}

	/**
	 * @see BceeSynchronizeJobSepaUeberweisung#getWebSite()
	 */
	@Override
	protected void getWebSite() throws WebSiteException, ApplicationException {
		webSite = BceeWebSite.getInstance(konto);
	}

	/**
	 * @see BceeSynchronizeJobSepaUeberweisung#login()
	 */
	@Override
	protected void login() throws WebSiteException, ApplicationException {
		final char[] password = PasswdCache.getPassword(Bcee.NAME, kundennummer);
		try {

			final String otp = BceeSynchronizeJobKontoauszugUtils.getOtp();
			webSite.login(password, otp);
		} catch (WebSiteException e) {
			if (e.getType().equals(WebSiteExceptionTypes.LOGIN_FAILED)) {
				// Passwort zurcksetzten
				PasswdCache.revoke(Bcee.NAME, kundennummer);
			}
			throw e;
		}
		Arrays.fill(password, ' ');

	}

	/**
	 * @see BceeSynchronizeJobSepaUeberweisung#handleGiroTransfer()
	 */
	@Override
	protected void handleGiroTransfer() throws WebSiteException, ApplicationException {

		try {
			webSite.createSepaTransfer(aUeberweisung);
			try {
				final String resultMsg = aUeberweisung.getMeta("resultMsg", null); //$NON-NLS-1$
				if (resultMsg != null) {
					monitor.setStatusText(resultMsg);
				}
			} catch (RemoteException e) {
				// Wird nur geloggt
				Logger.error(I18N.tr("BceeSynchronizeJobSepaUeberweisung.readMetaDataFailed"), e); //$NON-NLS-1$
			}
			markExecuted();

		} catch (WebSiteException | ApplicationException e) {
			markFailed(e.getMessage());
			throw e;
		}

	}

}
