//
// Copyright 2014 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.bcee.web;

import java.io.IOException;
import java.rmi.RemoteException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;

import com.gargoylesoftware.htmlunit.FailingHttpStatusCodeException;
import com.gargoylesoftware.htmlunit.ScriptResult;
import com.gargoylesoftware.htmlunit.WebClient;
import com.gargoylesoftware.htmlunit.html.DomElement;
import com.gargoylesoftware.htmlunit.html.HtmlDivision;
import com.gargoylesoftware.htmlunit.html.HtmlInput;
import com.gargoylesoftware.htmlunit.html.HtmlPage;

import de.frankmuenster.hibiscus.mahoe.bcee.Bcee;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.frankmuenster.hibiscus.mahoe.utils.Utils;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteException;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteExceptionTypes;
import de.willuhn.jameica.hbci.HBCIProperties;
import de.willuhn.jameica.hbci.rmi.AuslandsUeberweisung;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Statische Implementierungen der berweisungsfunktion
 * 
 * @author Frank Mnster
 * 
 */
public final class BceeWebSiteTransferImpl {

	/**
	 * Privater Konstruktor
	 */
	private BceeWebSiteTransferImpl() {
		throw new AssertionError("Instantiating utility class..."); //$NON-NLS-1$
	}

	/**
	 * Erstellt die berweisung bankseitig.
	 * 
	 * @param webClient
	 *            der WebClient, mit der angemeldeten Seite.
	 * @param aUeberweisung
	 *            die berweisung
	 * 
	 * @throws ApplicationException
	 * @throws WebSiteException
	 */
	public static void createSepaTransfer(final WebClient webClient, final AuslandsUeberweisung aUeberweisung)
			throws ApplicationException, WebSiteException {

		final HtmlPage page = openNewTransfer(webClient);
		fillValues(page, aUeberweisung);
		final HtmlPage ctrlPage = controlTransfer(page);
		final String resultMsg = confirmTransfer(ctrlPage);
		if (StringUtils.isNotEmpty(resultMsg)) {
			try {
				aUeberweisung.setMeta("resultMsg", resultMsg); //$NON-NLS-1$
			} catch (RemoteException e) {
				// Der Fehler wird nur gelogt.
				Logger.error(I18N.tr("BceeWebSiteTransferImpl.saveMetaDataFailed"), e); //$NON-NLS-1$
			}

		}

		try {
			webClient.getPage(Bcee.URL_EURO_TRANSFER);
		} catch (FailingHttpStatusCodeException | IOException e) {
			// Nicht schlimm
			Logger.debug(I18N.tr("BceeWebSiteTransferImpl.resetTransferPageFailed") + e.getMessage()); //$NON-NLS-1$
		}

	}

	/**
	 * Klickt den Besttigen-Button und analysiert das Ergebnis.
	 * 
	 * @param ctrlPage
	 * @return die Banknachricht im Erfolgsfall.
	 * 
	 * @throws WebSiteException
	 */
	private static String confirmTransfer(final HtmlPage ctrlPage) throws WebSiteException {

		final ScriptResult result = ctrlPage.executeJavaScript("document.getElementById('btnValidate').click()"); //$NON-NLS-1$
		final HtmlPage resultPage = (HtmlPage) result.getNewPage();
		Logger.debug(resultPage.asText());

		if (resultPage.asText().contains("nschte Dienstleistung ist momentan nicht verf")) { //$NON-NLS-1$
			// Upps, ganz massiver Fehler...
			final String msg = I18N.tr("BceeWebSiteTransferImpl.serviceUnavailable"); // NOPMD //$NON-NLS-1$
			Logger.error(msg);
			throw new WebSiteException(msg, WebSiteExceptionTypes.TRANSFER_FAILED);
		}

		// check result
		final List<Object> divs = resultPage.getByXPath("//div[@class='validation-summary-errors']"); //$NON-NLS-1$
		if (!divs.isEmpty() && divs.get(0) instanceof HtmlDivision) {
			final HtmlDivision div = (HtmlDivision) divs.get(0);
			// Ups da ist ein Fehler
			final String msg = div.asText();
			Logger.error(msg);
			throw new WebSiteException(msg, WebSiteExceptionTypes.TRANSFER_FAILED);
		}

		// Besttigungsdialog suchen
		String resultMsg = null;
		final List<Object> divs2 = resultPage.getByXPath("//div[@aria-describedby='alertPaymentValidated']"); //$NON-NLS-1$
		if (!divs2.isEmpty() && divs.get(0) instanceof HtmlDivision) {
			final DomElement alterText = resultPage.getElementById("alertPaymentValidated"); //$NON-NLS-1$
			final String msg = alterText.asText();
			Logger.info(msg);
			resultMsg = msg;
		}

		return resultMsg;
	}

	/**
	 * Klickt den berprfen-Button und analysiert das Ergebnis.
	 * 
	 * @param page
	 * @return die neue Seite, falls erfolgreich.
	 * @throws WebSiteException
	 */
	private static HtmlPage controlTransfer(final HtmlPage page) throws WebSiteException {

		final ScriptResult result = page.executeJavaScript("document.getElementById('btnCtrl').click()"); //$NON-NLS-1$
		final HtmlPage resultPage = (HtmlPage) result.getNewPage();
		Logger.debug(resultPage.asText());

		if (resultPage.asText().contains("nschte Dienstleistung ist momentan nicht verf")) { //$NON-NLS-1$
			// Upps, ganz massiver Fehler...
			final String msg = I18N.tr("BceeWebSiteTransferImpl.serviceUnavailable"); // NOPMD //$NON-NLS-1$
			Logger.error(msg);
			throw new WebSiteException(msg, WebSiteExceptionTypes.TRANSFER_FAILED);
		}

		// check result
		final List<Object> divs = resultPage.getByXPath("//div[@class='validation-summary-errors']"); //$NON-NLS-1$
		if (!divs.isEmpty() && divs.get(0) instanceof HtmlDivision) {
			final HtmlDivision div = (HtmlDivision) divs.get(0);
			// Ups da ist ein Fehler
			final String msg = div.asText();
			Logger.error(msg);
			throw new WebSiteException(msg, WebSiteExceptionTypes.TRANSFER_FAILED);
		}
		Logger.info(I18N.tr("BceeWebSiteTransferImpl.inputCheckOk")); //$NON-NLS-1$
		return resultPage;
	}

	private static void fillValues(final HtmlPage page, final AuslandsUeberweisung aUeberweisung)
			throws WebSiteException, ApplicationException {

		try {

			// Auftraggeberkonto
			final String auftragsKto = HBCIProperties.formatIban(aUeberweisung.getKonto().getIban());
			final HtmlInput ktoInput = (HtmlInput) page.getElementById("PrincipalAccount"); //$NON-NLS-1$
			ktoInput.setValueAttribute(auftragsKto);

			// Empfnger IBAN
			final String ibanValue = aUeberweisung.getGegenkontoNummer();
			final HtmlInput iban = (HtmlInput) (page.getElementById("AccountNumber")); //$NON-NLS-1$
			iban.setValueAttribute(ibanValue);

			// Empfnger Name
			final String benfNameValue = aUeberweisung.getGegenkontoName();
			final HtmlInput benfName = (HtmlInput) page.getElementById("BenefName"); //$NON-NLS-1$
			benfName.setValueAttribute(benfNameValue);

			// Datum
			if (aUeberweisung.isTerminUeberweisung()) {
				final SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy", Locale.GERMAN); //$NON-NLS-1$
				final Date transDate = aUeberweisung.getTermin();
				final String transfertDateVal = sdf.format(transDate);
				final String script = "document.getElementById('TransfertDate').value = '" + transfertDateVal + "'"; //$NON-NLS-1$ //$NON-NLS-2$
				final ScriptResult result = page.executeJavaScript(script);
				Logger.debug("ScriptResult: " + !ScriptResult.isFalse(result)); //$NON-NLS-1$
			}

			// Betrag
			final String amountValue = Utils.convertDoubleToString(aUeberweisung.getBetrag());
			final HtmlInput amount = (HtmlInput) page.getElementById("Amount"); //$NON-NLS-1$
			amount.setValueAttribute(amountValue);

			// Zweck1
			final String comm1Value = aUeberweisung.getZweck();
			final HtmlInput communication1 = (HtmlInput) page.getElementById("Communication1"); //$NON-NLS-1$
			communication1.setValueAttribute(comm1Value);

			// Zweck2
			final String comm2Value = aUeberweisung.getZweck2();
			if (StringUtils.isNotEmpty(comm2Value)) {
				final HtmlInput communication2 = (HtmlInput) page.getElementById("Communication2"); //$NON-NLS-1$
				communication2.setValueAttribute(comm2Value);
			}

			final String[] ewz = aUeberweisung.getWeitereVerwendungszwecke();
			if (ewz != null && ewz.length > 0) {
				// Zweck3
				final String comm3Value = ewz[0];
				if (StringUtils.isNotEmpty(comm2Value)) {
					final HtmlInput communication3 = (HtmlInput) page.getElementById("Communication3"); //$NON-NLS-1$
					communication3.setValueAttribute(comm3Value);
				}

				if (ewz.length > 1 && StringUtils.isNotEmpty(ewz[1])) {
					// Zweck4
					final HtmlInput communication4 = (HtmlInput) page.getElementById("Communication4"); //$NON-NLS-1$
					communication4.setValueAttribute(ewz[1]);
				}
			}

		} catch (RemoteException e) {
			final String msg = I18N.tr("BceeWebSiteTransferImpl.readTransferFieldsFailed"); // NOPMD //$NON-NLS-1$
			Logger.error(msg, e);
			throw new ApplicationException(msg, e);
		}

	}

	/**
	 * ffnet einen neuen EuroTranfer
	 * 
	 * @param webClient
	 *            der angemeldete WebClient.
	 * @return die Seite mit einer neuen berweisung
	 * @throws WebSiteException
	 */
	private static HtmlPage openNewTransfer(final WebClient webClient) throws WebSiteException {

		try {
			final HtmlPage page = webClient.getPage(Bcee.URL_EURO_TRANSFER);
			Logger.debug(page.asText());
			if (!page.asText().contains("Eurotransferts")) { //$NON-NLS-1$
				// Wenn man hier ist, wurde das Konto nicht gefunden
				Logger.error(I18N.tr("BceeWebSiteUtils.accountNotFound")); //$NON-NLS-1$
				throw new WebSiteException(I18N.tr("BceeWebSiteUtils.accountNotFound"), //$NON-NLS-1$
						WebSiteExceptionTypes.KONTO_NOT_FOUND);
			}
			return page;

		} catch (FailingHttpStatusCodeException | IOException e) {
			final WebSiteException wse = new WebSiteException(e.getMessage(), WebSiteExceptionTypes.PAGE_NOT_FOUND);
			wse.addSuppressed(e);
			throw wse; // NOPMD
		}

	}

}
