//
// Copyright 2017 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.bsh.json.accountdetails;

import java.util.List;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;

/**
 * Vertragdetails - Daten
 * 
 * @author Frank Mnster
 * 
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({ "contracttype", "contractphase", "coredata", "subsidy", "debit", "contract", "assignmentgoal",
		"transactions" })
public class AccountDetailData {

	/**
	 * Zuteilungsaussicht
	 */
	@JsonProperty("assignmentgoal")
	private transient AssignmentGoal assignmentGoal;

	/**
	 * Vertragsdaten
	 */
	@JsonProperty("contract")
	private transient Contract contract;

	/**
	 * Vertragsphase
	 */
	@JsonProperty("contractphase")
	private transient Integer contractPhase;

	/**
	 * Vertragstyp
	 */
	@JsonProperty("contracttype")
	private transient Integer contractType;

	/**
	 * Vertrags Coredaten
	 */
	@JsonProperty("coredata")
	private transient CoreData coreData;

	/**
	 * Einzge
	 */
	@JsonProperty("debit")
	private transient Debit debit;

	/**
	 * Staatliche F&ouml;rderung
	 */
	@JsonProperty("subsidy")
	private transient Subsidy subsidy;

	/**
	 * Ums&auml;tze
	 */
	@JsonProperty("transactions")
	private transient List<TransactionYear> transactionYear;

	/**
	 * Standardkonstruktor
	 */
	public AccountDetailData() {
		// Absichtlich leer
	}

	/**
	 * Liefert die unverbindliche Zuteilungsaussicht
	 * 
	 * @return die unverbindliche Zuteilungsaussicht
	 */
	@JsonProperty("assignmentgoal")
	public AssignmentGoal getAssignmentgoal() {
		return assignmentGoal;
	}

	/**
	 * Liefert die Vertagsdaten
	 * 
	 * @return die Vertagsdaten
	 */
	@JsonProperty("contract")
	public Contract getContract() {
		return contract;
	}

	/**
	 * Liefert die Vertragsphase
	 * 
	 * @return die Vertragsphase
	 */
	@JsonProperty("contractphase")
	public Integer getContractphase() {
		return contractPhase;
	}

	/**
	 * Liefert den Vertragstyp
	 * 
	 * @return den Vertragstyp
	 */
	@JsonProperty("contracttype")
	public Integer getContracttype() {
		return contractType;
	}

	/**
	 * Liefert die Vertrags Coredaten
	 * 
	 * @return die Vertrags Coredaten
	 */
	@JsonProperty("coredata")
	public CoreData getCoreData() {
		return coreData;
	}

	/**
	 * Liefert die Einzge
	 * 
	 * @return die Einzge
	 */
	@JsonProperty("debit")
	public Debit getDebit() {
		return debit;
	}

	/**
	 * Liefert die staatliche F&ouml;rderung
	 * 
	 * @return die staatliche F&ouml;rderung
	 */
	@JsonProperty("subsidy")
	public Subsidy getSubsidy() {
		return subsidy;
	}

	/**
	 * Liefert die Ums&auml;tze
	 * 
	 * @return eine Liste mit den Ums&auml;tze
	 */
	@JsonProperty("transactions")
	public List<TransactionYear> getTransactions() {
		return transactionYear;
	}

}
