//
// Copyright 2017 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.bsh.json.accountdetails;

import java.util.List;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;

/**
 * VertragDetails - CoreData (Vertragsdaten)
 * 
 * @author Frank Mnster
 * 
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({ "bsn", "vtn", "role", "owners", "subsidycharge" })
public class CoreData {

	/**
	 * Bausparnummer (BSN)
	 */
	@JsonProperty("bsn")
	private transient String bsn;

	/**
	 * Vertragsinhaber
	 */
	@JsonProperty("owners")
	private transient List<String> owners;

	/**
	 * Die Rolle im Vertrag
	 */
	@JsonProperty("role")
	private transient String role;

	/**
	 * Begnstigte Aufwendungen
	 */
	@JsonProperty("subsidycharge")
	private transient String subsidycharge;

	/**
	 * Die VTN
	 */
	@JsonProperty("vtn")
	private transient String vtn;

	/**
	 * Standardkonstruktor
	 */
	public CoreData() {
		// Absichtlich leer
	}

	/**
	 * Liefert die Bausparnummer
	 * 
	 * @return die Bausparnummer
	 */
	@JsonProperty("bsn")
	public String getBsn() {
		return bsn;
	}

	/**
	 * Liefert die Vertragsinhaber
	 * 
	 * @return ein List emit den Vertragsinhaber
	 */
	@JsonProperty("owners")
	public List<String> getOwners() {
		return owners;
	}

	/**
	 * Liefert die Rolle im Vertrag
	 * 
	 * @return die Rolle im Vertrag
	 */
	@JsonProperty("role")
	public String getRole() {
		return role;
	}

	/**
	 * Liefert die beg&uuml;nstigten Aufwendungen
	 * 
	 * @return die beg&uuml;nstigten Aufwendungen
	 */
	@JsonProperty("subsidycharge")
	public String getSubsidycharge() {
		return subsidycharge;
	}

	/**
	 * Liefert die VTN
	 * 
	 * @return die VTN
	 */
	@JsonProperty("vtn")
	public String getVtn() {
		return vtn;
	}

}
