//
// Copyright 2017 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.bsh.json.accountdetails;

import java.util.List;
import java.util.Locale;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;

/**
 * Vertragdetails Umsatzmonat
 * 
 * @author Frank Mnster
 * 
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({ "name", "fullname", "entries" })
public class TransactionMonth { // NOPMD

	/**
	 * Kurzform Monat
	 */
	@JsonProperty("name")
	private transient String name;

	/**
	 * Langform Monat
	 */
	@JsonProperty("fullname")
	private transient String fullname;

	/**
	 * Buchungen im Monat
	 */
	@JsonProperty("entries")
	private transient List<TransactionEntry> entries;

	/**
	 * Standardkonstruktor
	 */
	public TransactionMonth() {
		// Absichtlich leer
	}

	/**
	 * Liefert die Kurzform des Monats
	 * 
	 * @return die Kurzform des Monats
	 */
	@JsonProperty("name")
	public String getName() {
		return name;
	}

	/**
	 * Liefert die Langform des Monats
	 * 
	 * @return die Langform des Monats
	 */
	@JsonProperty("fullname")
	public String getFullname() {
		return fullname;
	}

	/**
	 * Liefert den Monat als zweistellige Zahl.
	 * 
	 * @return den Monat als zweistellige Zahl.
	 */
	public int getMonthAsInt() { // NOPMD
		int iMonat;
		switch (name.toUpperCase(Locale.GERMAN)) {
		case "JAN": //$NON-NLS-1$
			iMonat = 1;
			break;
		case "FEB": //$NON-NLS-1$
			iMonat = 2;
			break;
		case "MRZ": //$NON-NLS-1$
			iMonat = 3;
			break;
		case "APR": //$NON-NLS-1$
			iMonat = 4;
			break;
		case "MAI": //$NON-NLS-1$
			iMonat = 5;
			break;
		case "JUN": //$NON-NLS-1$
			iMonat = 6;
			break;
		case "JUL": //$NON-NLS-1$
			iMonat = 7;
			break;
		case "AUG": //$NON-NLS-1$
			iMonat = 8;
			break;
		case "SEP": //$NON-NLS-1$
			iMonat = 9;
			break;
		case "OKT": //$NON-NLS-1$
			iMonat = 10;
			break;
		case "NOV": //$NON-NLS-1$
			iMonat = 11;
			break;
		case "DEZ": //$NON-NLS-1$
			iMonat = 12;
			break;
		default:
			iMonat = 1;
			break;
		}

		return iMonat;

	}

	/**
	 * Liefert ein Liste mit dem Buchen in diesem Monat.
	 * 
	 * @return ein Liste mit dem Buchen des Monats.
	 */
	@JsonProperty("entries")
	public List<TransactionEntry> getEntries() {
		return entries;
	}

}
