//
// Copyright 2017 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.bsh.json.accountdetails;

import java.math.BigDecimal;
import java.util.List;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;

import de.frankmuenster.hibiscus.mahoe.bsh.json.deserializer.MoneyDeserializer;

/**
 * Vertragdetails - Umsatz
 * 
 * @author Frank Mnster
 * 
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({ "year", "saldo", "month" })
public class TransactionYear {

	/**
	 * Das Jahr der Buchungen
	 */
	@JsonProperty("year")
	private transient Integer year;

	/**
	 * Das Endsaldo in dem Jahr
	 */
	@JsonProperty("saldo")
	@JsonDeserialize(using = MoneyDeserializer.class)
	private transient BigDecimal saldo;

	/**
	 * Liste der Monate
	 */
	@JsonProperty("month")
	private transient List<TransactionMonth> month;

	/**
	 * Standardkonstruktor
	 */
	public TransactionYear() {
		// Absichlich leer
	}

	/**
	 * Liefert das Jahr
	 * 
	 * @return das Jahr
	 */
	@JsonProperty("year")
	public Integer getYear() {
		return year;
	}

	/**
	 * Liefert das Endsaldo des Jahres
	 * 
	 * @return das Endsaldo
	 */
	@JsonProperty("saldo")
	public BigDecimal getSaldo() {
		return saldo;
	}

	/**
	 * Liefert eine Liste aller Monate mit Umstzen.
	 * 
	 * @return die Liste aller Monate mit Umstzen.
	 */
	@JsonProperty("month")
	public List<TransactionMonth> getMonth() {
		return month;
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return String.format("TransactionYear [year=%s, saldo=%s, month=%s]", year, saldo, month); //$NON-NLS-1$
	}

}
