//
// Copyright 2017 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.bsh.web;

import java.rmi.RemoteException;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.hibiscus.mahoe.MahoeSynchronizeBackend;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.frankmuenster.hibiscus.mahoe.web.AbstractWebAccountInformation;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;
import de.willuhn.util.ProgressMonitor;

/**
 * Bean mit Kontoinformationen von der Webseite
 * 
 * @author Frank Mnster
 * @version 1.0
 */
public class BshWebAccountInformation extends AbstractWebAccountInformation {

	/**
	 * Die BSN Nummer
	 */
	private String bsn;

	/**
	 * Die Vertragsbuchstabe
	 */
	private String contractLetter;

	/**
	 * Die Vertragsnummer
	 */
	private String contractNumber;

	/**
	 * Der Vertragstyp
	 */
	private String type;

	/**
	 * Die VTN Nummer
	 */
	private String vtn;

	/**
	 * Standardkonstruktor
	 */
	public BshWebAccountInformation() {
		super();
	}

	/**
	 * @return the bsn
	 */
	public String getBsn() {
		return bsn;
	}

	/**
	 * @return the contractLetter
	 */
	public String getContractLetter() {
		return contractLetter;
	}

	/**
	 * @return the contractNumber
	 */
	public String getContractNumber() {
		return contractNumber;
	}

	/**
	 * @return the type
	 */
	public String getType() {
		return type;
	}

	/**
	 * @return the vtn
	 */
	public String getVtn() {
		return vtn;
	}

	/**
	 * @param bsn
	 *            the bsn to set
	 */
	public void setBsn(final String bsn) {
		this.bsn = bsn;
	}

	/**
	 * @param contractLetter
	 *            the contractLetter to set
	 */
	public void setContractLetter(final String contractLetter) {
		this.contractLetter = contractLetter;
	}

	/**
	 * @param contractNumber
	 *            the contractNumber to set
	 */
	public void setContractNumber(final String contractNumber) {
		this.contractNumber = contractNumber;
	}

	/**
	 * @param type
	 *            the type to set
	 */
	public void setType(final String type) {
		this.type = type;
	}

	/**
	 * @param vtn
	 *            the vtn to set
	 */
	public void setVtn(final String vtn) {
		this.vtn = vtn;
	}

	/**
	 * @see AbstractWebAccountInformation#saveBankSpecificInfo(de.willuhn.util.ProgressMonitor)
	 */
	@Override
	protected void saveBankSpecificInfo(final ProgressMonitor monitor) throws ApplicationException {

		// vtn-Nummer
		if (vtn != null) {
			monitor.setStatusText(I18N.tr("BshWebAccountInformation.saveVtn")); //$NON-NLS-1$
			try {
				final String ktoVTN = konto.getMeta(MahoeSynchronizeBackend.BSH_VTN, null);
				if (StringUtils.isEmpty(ktoVTN)) {
					konto.setMeta(MahoeSynchronizeBackend.BSH_VTN, vtn);
				}
			} catch (RemoteException e) {
				Logger.error(I18N.tr(I18N.tr("BshWebAccountInformation.saveVtnFailed")), e); //$NON-NLS-1$
				throw new ApplicationException(I18N.tr("BshWebAccountInformation.saveVtnFailed"), e); //$NON-NLS-1$
			}
		}

		// Unterkonto
		final StringBuilder unterKto = new StringBuilder();
		unterKto.append(getContractLetter()).append(getContractNumber()).append('%').append(getType());
		try {
			if (!unterKto.equals(konto.getUnterkonto())) {
				unterKto.setLength(30);
				konto.setUnterkonto(unterKto.toString());
			}
		} catch (RemoteException e) {
			Logger.error(I18N.tr(I18N.tr("BshWebAccountInformation.saveSubAccountFailed")), e); //$NON-NLS-1$
			throw new ApplicationException(I18N.tr("BshWebAccountInformation.saveSubAccountFailed"), e); //$NON-NLS-1$
		}
	}
}
