//
// Copyright 2016 frankmuenster.de
//
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
//
//   http://ec.europa.eu/idabc/eupl
//
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//

package de.frankmuenster.hibiscus.mahoe.pdfextractor;

import java.io.IOException;
import java.util.List;

import org.apache.pdfbox.text.PDFTextStripperByArea;
import org.apache.pdfbox.text.TextPosition;

/**
 * Extended PDFTextStripperByArea class to add positions of each word to the
 * result string.
 *
 * @author Frank Mnster
 *
 */
public abstract class AbstractBankPDFPage extends PDFTextStripperByArea {

	/**
	 * Conversion value from CM to PDFBox values
	 */
	protected final static double CM_TO_PDF = 28.34645669;

	/**
	 * Area-Name fr das Kartenkonto
	 */
	public final static String KARTEN_KONTO = "kartenkonto"; //$NON-NLS-1$

	/**
	 * Area-Name fr den Karteninhaber
	 */
	public static final String KARTEN_INHABER = "karteninhabero"; //$NON-NLS-1$

	/**
	 * Area-Name fr Rechungsdatum
	 */
	public final static String RECHNUNGS_DATUM = "rechnungsdatum"; //$NON-NLS-1$

	/**
	 * Area-Name fr Saldo
	 */
	public final static String SALDO = "saldo"; //$NON-NLS-1$

	/**
	 * Area-Name fr Buchungsdatum
	 */
	public final static String BUCHUNGS_DATUM = "buchungsdatum"; //$NON-NLS-1$

	/**
	 * Area-Name fr das Kaufdatum
	 */
	public final static String KAUF_DATUM = "kaufdatum"; //$NON-NLS-1$

	/**
	 * Area-Name fr die Beschreibung
	 */
	public final static String BESCHREIBUNG = "beschreibung"; //$NON-NLS-1$

	/**
	 * Area-Name fr die Fremdwhrung
	 */
	public final static String FREMDWAEHRUNG = "fremdwaehrung"; //$NON-NLS-1$

	/**
	 * Area-Name fr den Fremdwhrungsbetrag
	 */
	public final static String FREMDW_BETRAG = "fremdwaehrungsbetrag"; //$NON-NLS-1$

	/**
	 * Area-Name fr denKurs
	 */
	public final static String KURS = "kurs"; //$NON-NLS-1$

	/**
	 * Area-Name fr den Betrag
	 */
	public final static String BETRAG = "betrag"; //$NON-NLS-1$

	/**
	 * Area-Name fr den f&auml;lligen Betrag
	 */
	public static final String FAELLIG_BETRAG = "faelligbetrag"; //$NON-NLS-1$

	/**
	 * Area-Name fr das F&auml;lligkeitsdatum
	 */
	public static final String FAELLIG_DATUM = "faelligdatum"; //$NON-NLS-1$

	/**
	 * Area-Name fr die Empfnger IBAN der Ausgleichsbuchung
	 */
	public static final String FAELLIG_IBAN = "faelligiban"; //$NON-NLS-1$

	/**
	 * Area-Name fr die Empfnger BIC der Ausgleichsbuchung
	 */
	public static final String FAELLIG_BIC = "faelligbic"; //$NON-NLS-1$

	/**
	 * Standard constructor call super class.
	 *
	 * @throws IOException
	 */
	public AbstractBankPDFPage() throws IOException {
		super();
		this.setSortByPosition(true);
		this.setEndPage(1);
		this.setRegions();
	}

	/**
	 * Setzt die Regionen auf der Seite
	 */
	protected abstract void setRegions();

	/**
	 * Liefert die zu erwartete X-Position des Buchungsdaten
	 *
	 * @return die zu erwartete X-Position des Buchungsdaten
	 */
	protected abstract double getXPos();

	/**
	 * Override the default functionality of PDFTextStripper.
	 */
	@Override
	protected void writeString(final String text, final List<TextPosition> textPositions) throws IOException {
		final TextPosition firstPosition = textPositions.get(0);
		this.writeString(String.format("%s#*#%s#*#%s", firstPosition.getTextMatrix() //$NON-NLS-1$
				.getTranslateX(), firstPosition.getTextMatrix().getTranslateY(), text));

	}

}
