//
// Copyright 2016 frankmuenster.de
//
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
//
//   http://ec.europa.eu/idabc/eupl
//
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//

package de.frankmuenster.hibiscus.mahoe.santander.pdfextractor;

import java.awt.Rectangle;
import java.io.IOException;

import de.frankmuenster.hibiscus.mahoe.pdfextractor.AbstractBankPDFPage;

/**
 * TextAreas fr die erste Seite
 *
 * @author Frank Mnster
 */
public class SantanderPdfFirstPage extends AbstractBankPDFPage {

	/**
	 * Erwartete X-Position des Buchungsdaten
	 */
	private final static double BUCH_DATUM_X_POS = 33.35;

	/**
	 * Standard constructor call super class.
	 *
	 * @throws IOException
	 */
	public SantanderPdfFirstPage() throws IOException {
		super();
	}

	/**
	 * @see AbstractBankPDFPage#setRegions()
	 */
	@Override
	@SuppressWarnings("PMD.ExcessiveMethodLength")
	protected final void setRegions() {

		int areaX;
		int areaY;
		int areaW;
		int areaH;
		Rectangle rect = new Rectangle(10, 10, 10, 10);

		// Kartenkonto rectangle
		areaX = (int) (12.5 * AbstractBankPDFPage.CM_TO_PDF);
		areaY = (int) (3.3 * AbstractBankPDFPage.CM_TO_PDF);
		areaW = (int) (4 * AbstractBankPDFPage.CM_TO_PDF);
		areaH = (int) (0.6 * AbstractBankPDFPage.CM_TO_PDF);
		rect = new Rectangle(areaX, areaY, areaW, areaH);
		this.addRegion(AbstractBankPDFPage.KARTEN_KONTO, rect);

		// Karteninhaber rectangle
		areaX = (int) (12.5 * AbstractBankPDFPage.CM_TO_PDF);
		areaY = (int) (4.3 * AbstractBankPDFPage.CM_TO_PDF);
		areaW = (int) (4 * AbstractBankPDFPage.CM_TO_PDF);
		areaH = (int) (0.6 * AbstractBankPDFPage.CM_TO_PDF);
		rect = new Rectangle(areaX, areaY, areaW, areaH);
		this.addRegion(AbstractBankPDFPage.KARTEN_INHABER, rect);

		// Rechnungsdatum rectangle
		areaX = (int) (17 * AbstractBankPDFPage.CM_TO_PDF);
		areaY = (int) (6.5 * AbstractBankPDFPage.CM_TO_PDF);
		areaW = (int) (2 * AbstractBankPDFPage.CM_TO_PDF);
		areaH = (int) (0.5 * AbstractBankPDFPage.CM_TO_PDF);
		rect = new Rectangle(areaX, areaY, areaW, areaH);
		this.addRegion(AbstractBankPDFPage.RECHNUNGS_DATUM, rect);

		// Saldo rectangle
		areaX = (int) (17 * AbstractBankPDFPage.CM_TO_PDF);
		areaY = (int) (7.6 * AbstractBankPDFPage.CM_TO_PDF);
		areaW = (int) (3 * AbstractBankPDFPage.CM_TO_PDF);
		areaH = (int) (0.8 * AbstractBankPDFPage.CM_TO_PDF);
		rect = new Rectangle(areaX, areaY, areaW, areaH);
		this.addRegion(AbstractBankPDFPage.SALDO, rect);

		// Flliger Betrag rectangle
		areaX = (int) (17 * AbstractBankPDFPage.CM_TO_PDF);
		areaY = (int) (8.3 * AbstractBankPDFPage.CM_TO_PDF);
		areaW = (int) (3 * AbstractBankPDFPage.CM_TO_PDF);
		areaH = (int) (0.8 * AbstractBankPDFPage.CM_TO_PDF);
		rect = new Rectangle(areaX, areaY, areaW, areaH);
		this.addRegion(AbstractBankPDFPage.FAELLIG_BETRAG, rect);

		// Flligkeitsdaum rectangle
		areaX = (int) (17 * AbstractBankPDFPage.CM_TO_PDF);
		areaY = (int) (8.9 * AbstractBankPDFPage.CM_TO_PDF);
		areaW = (int) (3 * AbstractBankPDFPage.CM_TO_PDF);
		areaH = (int) (0.8 * AbstractBankPDFPage.CM_TO_PDF);
		rect = new Rectangle(areaX, areaY, areaW, areaH);
		this.addRegion(AbstractBankPDFPage.FAELLIG_DATUM, rect);

		// Flliger Betrag Empfnger IBAN rectangle
		areaX = (int) (14.5 * AbstractBankPDFPage.CM_TO_PDF);
		areaY = (int) (27.8 * AbstractBankPDFPage.CM_TO_PDF);
		areaW = (int) (8 * AbstractBankPDFPage.CM_TO_PDF);
		areaH = (int) (0.3 * AbstractBankPDFPage.CM_TO_PDF);
		rect = new Rectangle(areaX, areaY, areaW, areaH);
		this.addRegion(AbstractBankPDFPage.FAELLIG_IBAN, rect);

		// Flliger Betrag Empfnger BIC rectangle
		areaX = (int) (14.5 * AbstractBankPDFPage.CM_TO_PDF);
		areaY = (int) (28.1 * AbstractBankPDFPage.CM_TO_PDF);
		areaW = (int) (8 * AbstractBankPDFPage.CM_TO_PDF);
		areaH = (int) (0.3 * AbstractBankPDFPage.CM_TO_PDF);
		rect = new Rectangle(areaX, areaY, areaW, areaH);
		this.addRegion(AbstractBankPDFPage.FAELLIG_BIC, rect);

		// Buchungsdatum rectangle
		areaX = (int) (1.1 * AbstractBankPDFPage.CM_TO_PDF);
		areaY = (int) (15 * AbstractBankPDFPage.CM_TO_PDF);
		areaW = (int) (1.5 * AbstractBankPDFPage.CM_TO_PDF);
		areaH = (int) (11 * AbstractBankPDFPage.CM_TO_PDF);
		rect = new Rectangle(areaX, areaY, areaW, areaH);
		this.addRegion(AbstractBankPDFPage.BUCHUNGS_DATUM, rect);

		// Kaufdatum rectangle
		areaX = (int) (16.2 * AbstractBankPDFPage.CM_TO_PDF);
		areaY = (int) (15 * AbstractBankPDFPage.CM_TO_PDF);
		areaW = (int) (1.4 * AbstractBankPDFPage.CM_TO_PDF);
		areaH = (int) (11 * AbstractBankPDFPage.CM_TO_PDF);
		rect = new Rectangle(areaX, areaY, areaW, areaH);
		this.addRegion(AbstractBankPDFPage.KAUF_DATUM, rect);

		// Beschreibung rectangle
		areaX = (int) (2.9 * AbstractBankPDFPage.CM_TO_PDF);
		areaY = (int) (15 * AbstractBankPDFPage.CM_TO_PDF);
		areaW = (int) (9.7 * AbstractBankPDFPage.CM_TO_PDF);
		areaH = (int) (11 * AbstractBankPDFPage.CM_TO_PDF);
		rect = new Rectangle(areaX, areaY, areaW, areaH);
		this.addRegion(AbstractBankPDFPage.BESCHREIBUNG, rect);

		// Fremdwhrungsbetrag rectangle
		areaX = (int) (12.6 * AbstractBankPDFPage.CM_TO_PDF);
		areaY = (int) (15 * AbstractBankPDFPage.CM_TO_PDF);
		areaW = (int) (1.9 * AbstractBankPDFPage.CM_TO_PDF);
		areaH = (int) (11 * AbstractBankPDFPage.CM_TO_PDF);
		rect = new Rectangle(areaX, areaY, areaW, areaH);
		this.addRegion(AbstractBankPDFPage.FREMDW_BETRAG, rect);

		// Fremdwhrung rectangle ???
		areaX = (int) (30.9 * AbstractBankPDFPage.CM_TO_PDF);
		areaY = (int) (12.6 * AbstractBankPDFPage.CM_TO_PDF);
		areaW = (int) (1.9 * AbstractBankPDFPage.CM_TO_PDF);
		areaH = (int) (11 * AbstractBankPDFPage.CM_TO_PDF);
		rect = new Rectangle(areaX, areaY, areaW, areaH);
		this.addRegion(AbstractBankPDFPage.FREMDWAEHRUNG, rect);

		// Kurs rectangle
		areaX = (int) (14.6 * AbstractBankPDFPage.CM_TO_PDF);
		areaY = (int) (15 * AbstractBankPDFPage.CM_TO_PDF);
		areaW = (int) (1.4 * AbstractBankPDFPage.CM_TO_PDF);
		areaH = (int) (11 * AbstractBankPDFPage.CM_TO_PDF);
		rect = new Rectangle(areaX, areaY, areaW, areaH);
		this.addRegion(AbstractBankPDFPage.KURS, rect);

		// Betrag rectangle
		areaX = (int) (17.8 * AbstractBankPDFPage.CM_TO_PDF);
		areaY = (int) (15 * AbstractBankPDFPage.CM_TO_PDF);
		areaW = (int) (2.1 * AbstractBankPDFPage.CM_TO_PDF);
		areaH = (int) (11 * AbstractBankPDFPage.CM_TO_PDF);
		rect = new Rectangle(areaX, areaY, areaW, areaH);
		this.addRegion(AbstractBankPDFPage.BETRAG, rect);

	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.pdfextractor.AbstractBankPDFPage#getXPos()
	 */
	@Override
	protected double getXPos() {
		return SantanderPdfFirstPage.BUCH_DATUM_X_POS;
	}

}
