//
// Copyright 2017 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.santander.web;

import java.io.IOException;
import java.rmi.RemoteException;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import com.gargoylesoftware.htmlunit.BrowserVersion;
import com.gargoylesoftware.htmlunit.ElementNotFoundException;
import com.gargoylesoftware.htmlunit.FailingHttpStatusCodeException;
import com.gargoylesoftware.htmlunit.NicelyResynchronizingAjaxController;
import com.gargoylesoftware.htmlunit.WebClient;
import com.gargoylesoftware.htmlunit.WebClientOptions;
import com.gargoylesoftware.htmlunit.html.DomElement;
import com.gargoylesoftware.htmlunit.html.HtmlForm;
import com.gargoylesoftware.htmlunit.html.HtmlInput;
import com.gargoylesoftware.htmlunit.html.HtmlPage;
import com.gargoylesoftware.htmlunit.html.HtmlPasswordInput;
import com.gargoylesoftware.htmlunit.html.HtmlSubmitInput;

import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.frankmuenster.hibiscus.mahoe.utils.Utils;
import de.frankmuenster.hibiscus.mahoe.utils.pdfsaver.PdfSaverException;
import de.frankmuenster.hibiscus.mahoe.web.AbstractWebSite;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteException;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteExceptionTypes;
import de.willuhn.jameica.hbci.rmi.AuslandsUeberweisung;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.KontoType;
import de.willuhn.jameica.hbci.rmi.Umsatz;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Santander Bank Website
 * 
 * @author Frank Mnster
 * 
 * @param <I>
 *            SantanderWebAccountInformation
 * @param <M>
 *            SantanderBankMessages
 * 
 */
@SuppressWarnings({ "PMD.ExcessiveImports" })
public abstract class AbstractSantanderWebSite<I extends SantanderWebAccountInformation, M extends SantanderBankMessages>
		extends AbstractWebSite<I, M> {

	/**
	 * Unsere BceeWebSite instance
	 */
	private static AbstractSantanderWebSite<SantanderWebAccountInformation, SantanderBankMessages> webSite;

	/**
	 * Das Benutzername
	 */
	private final transient String userName;

	/**
	 * Liefert die Instance der WebSeite
	 * 
	 * @param konto
	 * 
	 * @return die WebSeite
	 * 
	 * @throws WebSiteException
	 * @throws ApplicationException
	 */
	public static AbstractSantanderWebSite<SantanderWebAccountInformation, SantanderBankMessages> getInstance(
			final Konto konto) throws WebSiteException, ApplicationException {
		String userName = null;
		KontoType type = null;
		try {
			userName = konto.getKundennummer();
			type = KontoType.find(konto.getAccountType());

		} catch (RemoteException e) {
			final String msg = I18N.tr("AbstractSantanderWebSite.loginMissing"); //$NON-NLS-1$
			Logger.error(msg, e);
			throw new ApplicationException(msg, e);
		}

		if (AbstractSantanderWebSite.webSite == null) { // NOPMD
			if (KontoType.GIRO.equals(type)) {
				AbstractSantanderWebSite.webSite = new SantanderWebSiteGiro(konto, userName);
			} else if (KontoType.KREDITKARTE.equals(type)) {
				AbstractSantanderWebSite.webSite = new SantanderWebSiteKreditkarte(konto, userName);
			} else {
				throw new ApplicationException(I18N.tr("AbstractSantanderWebSite.missingAccountType")); //$NON-NLS-1$
			}
		} else {
			// Ist der Benutzername und der Kontotyp derselbe
			if (AbstractSantanderWebSite.webSite.userName.equals(userName)
					&& AbstractSantanderWebSite.webSite.isSupporting(type)) {
				// Aktualisiere Startdatum der Umsatzabfrage
				final Date lastTransacDate = Utils.getLastUmsatzDate(konto,
						AbstractSantanderWebSite.webSite.getStartdate());
				AbstractSantanderWebSite.webSite.accountInfo.setLastTransactionDate(lastTransacDate);
			} else {
				AbstractSantanderWebSite.webSite.logoff();
				if (KontoType.GIRO.equals(type)) {
					AbstractSantanderWebSite.webSite = new SantanderWebSiteGiro(konto, userName);
				} else if (KontoType.KREDITKARTE.equals(type)) {
					AbstractSantanderWebSite.webSite = new SantanderWebSiteKreditkarte(konto, userName);
				} else {
					throw new ApplicationException(I18N.tr("AbstractSantanderWebSite.missingAccountType")); //$NON-NLS-1$
				}
			}
		}
		return AbstractSantanderWebSite.webSite;
	}

	/**
	 * &Uuml;perf&uuml;pt, ob die WebSeiteninstance den Kontotypen
	 * unterst&uuml;tzt.
	 * 
	 * @param type
	 *            der Kontoyp
	 * @return <i>true</i>, wenn der Kontotyp unterst&uuml;tzt wird,
	 *         <i>false</i> sonst.
	 */
	protected abstract boolean isSupporting(KontoType type);

	/**
	 * Erstellt ein neues Webseiten-Objekt zur Kontaktaufnahme mit der BCEE.
	 * 
	 * @param konto
	 * @param userName
	 * 
	 * @throws WebSiteException
	 * 
	 */
	protected AbstractSantanderWebSite(final Konto konto, final String userName) throws WebSiteException {
		super(konto);
		this.userName = userName;

		this.accountInfo.setBankType(SantanderBankEnum.find(this.accountInfo.getKonto()));
		this.webClient = this.getWebClient();

	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.web.WebSite#createSepaTransfer(de.willuhn.jameica.hbci.rmi.AuslandsUeberweisung)
	 */
	@Override
	public void createSepaTransfer(final AuslandsUeberweisung aUeberweisung)
			throws WebSiteException, ApplicationException {
		Logger.info(I18N.tr("AbstractSantanderWebSite.transferNotSupported")); //$NON-NLS-1$
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.web.WebSite#getAccountInformation()
	 */
	@Override
	public abstract I getAccountInformation() throws WebSiteException, ApplicationException;

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.web.WebSite#getBookedTransactions()
	 */
	@Override
	public abstract List<Umsatz> getBookedTransactions() throws WebSiteException, ApplicationException;

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.web.WebSite#getPreBookedTransactions()
	 */
	@Override
	public abstract List<Umsatz> getPreBookedTransactions() throws WebSiteException, ApplicationException;

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.web.WebSite#isStillLoggedIn()
	 */
	@Override
	public boolean isStillLoggedIn() throws WebSiteException {
		// Brauchen wir noch nicht, Wird halt neu angemeldet.
		return false;
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.web.WebSite#login(char[],
	 *      java.lang.String[])
	 * 
	 * @param loginParams
	 *            muss Login_URL enthalten
	 */
	@Override
	public void login(final char[] password, final String... loginParams) throws WebSiteException {

		if (loginParams.length != 0) {
			throw new WebSiteException(I18N.tr("AbstractSantanderWebSite.loginParameterNumberWrong"), //$NON-NLS-1$
					WebSiteExceptionTypes.LOGIN_ERROR);
		}

		// Login URL
		final String loginURL = this.accountInfo.getBankType().getLoginUrl();

		try {
			final HtmlPage pageLogin = this.webClient.getPage(loginURL);

			final DomElement divContainer = pageLogin.getElementById("content"); //$NON-NLS-1$
			final HtmlForm formLogin = divContainer.getFirstByXPath("//div[1]/div[2]/form"); //$NON-NLS-1$

			Logger.debug(formLogin.asText());

			final HtmlInput inputUser = formLogin.getInputByName("loginDataForm.personNumber"); //$NON-NLS-1$
			final HtmlPasswordInput inputPasswd = formLogin.getInputByName("loginDataForm.PIN"); //$NON-NLS-1$
			final HtmlInput submitContinue = formLogin.getInputByValue("Login"); //$NON-NLS-1$

			// Setze Benutzernamen
			Logger.debug("Setzte Kundenummer auf: ****" + this.userName.substring(this.userName.length() - 4)); //$NON-NLS-1$
			inputUser.setValueAttribute(this.userName);
			// Setze Passwort
			inputPasswd.setValueAttribute(new String(password));

			// Invoke Click
			final HtmlPage pageContent = submitContinue.click();

			// And again test for error
			if (pageContent.asText().contains("Letzter Login") //$NON-NLS-1$
					|| pageContent.asText().contains("Letzte erfolgreiche Anmeldung")) { //$NON-NLS-1$
				Logger.info(I18N.tr("AbstractSantanderWebSite.loginSuccessful")); //$NON-NLS-1$
			} else {
				Logger.error(I18N.tr("AbstractSantanderWebSite.loginFailed")); //$NON-NLS-1$
				throw new WebSiteException(I18N.tr("AbstractSantanderWebSite.loginFailed"), //$NON-NLS-1$
						WebSiteExceptionTypes.LOGIN_ERROR);
			}

		} catch (FailingHttpStatusCodeException | IOException e) {
			Logger.error(I18N.tr("AbstractSantanderWebSite.loginFailedLog", e.getMessage()), e); //$NON-NLS-1$
			throw new WebSiteException(I18N.tr("AbstractSantanderWebSite.openLoginPageFailed"), //$NON-NLS-1$
					WebSiteExceptionTypes.LOGIN_ERROR, e);
		}
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.web.WebSite#logoff()
	 */
	@Override
	public void logoff() throws WebSiteException {

		try {

			HtmlPage page = this.webClient.getPage(this.accountInfo.getBankType().getOverviewUrl());
			final HtmlForm logoffForm = page.getFormByName("LOGOFF"); //$NON-NLS-1$
			final HtmlSubmitInput submit = logoffForm.getInputByValue("Abmelden"); //$NON-NLS-1$
			page = submit.click();

			Logger.debug(page.asText());
			if (!page.asText().contains("Sie wurden erfolgreich vom Online Banking abgemeldet.")) { //$NON-NLS-1$
				Logger.error(I18N.tr("AbstractSantanderWebSite.loggoutFailed")); //$NON-NLS-1$
			}

		} catch (ElementNotFoundException | FailingHttpStatusCodeException | IOException e) {
			// Nur Loggen
			Logger.error(I18N.tr("AbstractSantanderWebSite.loggoutFailed"), e); //$NON-NLS-1$
		}

	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.web.WebSite#savePDF(de.frankmuenster.hibiscus.mahoe.web.WebAccountInformation)
	 */
	@Override
	public abstract int savePDF(final SantanderWebAccountInformation accountInfo) throws PdfSaverException;

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.web.AbstractWebSite#getStartdate()
	 */
	@Override
	protected abstract Calendar getStartdate();

	/**
	 * Liefert einen instanzierten HtmlUnit WebClient
	 * 
	 * @return der WebClient
	 */
	@Override
	public final WebClient getWebClient() {

		if (this.webClient == null) {

			this.webClient = new WebClient(BrowserVersion.FIREFOX_45);

			final WebClientOptions options = this.webClient.getOptions();

			options.setUseInsecureSSL(false);
			options.setJavaScriptEnabled(true);
			options.setThrowExceptionOnFailingStatusCode(false);
			options.setCssEnabled(true);
			options.setThrowExceptionOnScriptError(false);
			options.setRedirectEnabled(true);

			this.webClient.setAjaxController(new NicelyResynchronizingAjaxController());
			this.webClient.getCookieManager().setCookiesEnabled(true);

			Utils.setProxyCfg(this.webClient, SantanderBankEnum.CONSUMER.getLoginUrl());

		}
		return this.webClient;

	}
}
